import "reflect-metadata";
import Container from "typedi";
import { ApolloServer } from "apollo-server";
import { useContainer, buildSchema, formatArgumentValidationError } from "../../src";

import { RecipeResolver } from "./recipe/recipe.resolver";
import { ResolveTimeMiddleware } from "./middlewares/resolve-time";
import { ErrorLoggerMiddleware } from "./middlewares/error-logger";

async function bootstrap() {
  useContainer(Container);

  // build TypeGraphQL executable schema
  const schema = await buildSchema({
    resolvers: [RecipeResolver],
    globalMiddlewares: [ErrorLoggerMiddleware, ResolveTimeMiddleware],
  });

  // Create GraphQL server
  const server = new ApolloServer({ schema, formatError: formatArgumentValidationError });

  // Start the server
  const { url } = await server.listen(4000);
  console.log(`Server is running, GraphQL Playground available at ${url}`);
}

bootstrap();
